#!/usr/bin/perl -w

# INPUT: xxx.fas  a fasta file with short sequences in it. Microsatellites are printed in lowercase letters

# makes a all agints all blast with soft filtering '-F m -U T -m 8'
# eliminates reciproc hits, autohits and multiple hits between the two same sequence
# takes only 100 % hits, eliminate fragments that are completely covered by another fragment
# concatanates overlaping sequences (only 100% ident) (in a cycle, till there is not 100% cover or overlap left)
# concataneted sequences are blasted againts each other and original non-red sequences, to detect hits.

#All hits are printed into xxx_short_final.tbl (without redundant sequences but with new concat sequences, no autohits, no reciproc hits and no multiple hits)

# OUTPUT: 	xxx_short_final.tbl hits to be treated (without redundant sequences but with new concat sequences, no autohits, no reciproc hits and no multiple hits)
# 		xxx_redundant.tbl 
#			first column all redundant sequences (even intermediates in concatenation steps)
#			second column;the longest seq that covers completely with 100% identity the first seq
#		xxx_multihit_final.tbl	all multihits (between two given seq) between non_red sequences
#		xxx_nr.fas	fasta file with all non-red sequences including original and concatenated ones
#		xxx_concat_info.txt file with each concatenated sequences code in the first column, 
#			the two merged sequenced code in the second and third columns

  use warnings;
  use strict;
  use subprogramEM;


my $e = '';
my $blast_path = '';
my $syst = '';
my $folder = '';
my $outfolder = '';
my $file_mot = '';

if ( scalar@ARGV == 6)
{
	$e = $ARGV[0];
	$blast_path = $ARGV[1];
	$syst = $ARGV[2];
	$folder = $ARGV[3];
	$outfolder = $ARGV[4];
	$file_mot = $ARGV[5];
}
else
{
	$e = '1E-40';
	#$blast_path = '/home/user0/bin/blast-2.2.19/bin/';
	#$blast_path = '/usr/local/bin/blast-2.2.13/bin/';
	$blast_path = 'c:/BLAST2_2_18/bin/';
	$syst = 'win';
	#$syst = 'linux';
	$folder = 'datatemp1';
	$outfolder = 'temp1';
	$file_mot = '_mask\.fas';
}



  unless ( opendir(FOLDER, $folder) )
  {
      print "Cannot access to folder $folder\n";
      exit;
  }
my @filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);




foreach my $filename (@filenames)
{
if ($filename =~ /$file_mot$/)
{
	my $blast_out = $filename;
	$blast_out =~ s/\.fas/_m8\.tbl/;
	$blast_out = $outfolder.'/'.$blast_out;
	my $fasta_nr = $filename;
	$fasta_nr =~ s/\./_nr\./;
	$fasta_nr = $outfolder.'/'.$fasta_nr;
#	print "$fasta_nr\n";
	my $red_file = $filename;
	$red_file =~ s/\.fas/_redundant\.tbl/;
	$red_file = $outfolder.'/'.$red_file;
	my $multihit_final = $filename;
	$multihit_final =~ s/\.fas/_multihit_final\.tbl/;
	$multihit_final = $outfolder.'/'.$multihit_final;
	my %concat_info = ();
	my $concat_info_file = $filename;
	$concat_info_file =~ s/\.fas/_concat_info\.txt/;
	$concat_info_file = $outfolder.'/'.$concat_info_file;
	my $concat_count = 0;

      $filename = $folder.'/'.$filename;
##############################################""
my $file_bool = check_if_first_line_empty($filename);
   call_blast_tbl_soft_filter($filename, $blast_out, $e, $blast_path);
	if ($syst eq 'win' and $file_bool ==1)
	{
	    delete_blast_temp_files_win1($folder);
	}
	elsif ($syst eq 'linux' and $file_bool ==1)
	{
	    delete_blast_temp_files($folder);
	}
###############################################
   	my @perc100 = ();
	my @group = ();
	my $multihit = $blast_out;
	$multihit =~ s/m8/multihit/;
	my $blast_short = $blast_out;
	$blast_short =~ s/m8/short/;
	my $blast_non_rec = $blast_out;
	$blast_non_rec =~ s/m8/non_rec/;

	eliminate_reciproc_hits($blast_out, $blast_non_rec);

# look at each line of the blast; put counter into a table if multiple hit.
# Then go through file agin to eliminate autohits, multiple hits
# print file with only informatif lines (no rec, auto or multiple) and define @perc100 with lines that have 100 ident.

	keep_info_lines_make_perc100($blast_non_rec, $blast_short, $multihit, \@perc100);

#-----------------------------------------------
#sort out 100 identity lines
	my %seq = read_hach_from_fasta_code_wo_space($filename);
      my %seq_l = length_sequences(\%seq);
	my %red = ();
	my @overlap = ();

# make hach with short-long complete cover %red
# put perfect overlap into new table @overlap 
	define_cover_red_overlap_table(\@perc100, \%seq_l, \%red, \@overlap);
# for each redundant find the longest idem
        find_longest_red(\%red);

#	print_hachage(\%red);

# eliminates lines in @overap if redundant seq is included
#print "@overlap\n";
eliminate_red_from_overlap(\@overlap, \%red);
# while there are lines in overlap, concat sequences if none of the pair is red, 
# makes new file with concat seq, and the noncatanated, but overlaping sequences, and blast, sort redundant, make new @overlap
concat_overlap_in_cycle(\%concat_info, \$concat_count, \@overlap, \%red, \%seq, \%seq_l, $outfolder, $blast_out, $multihit, $blast_short, $blast_non_rec, $e, $blast_path);

# fasta file (orig.fas) with all non redundant original sequences and concat.fas with all concatenated non-red sequences
# fasta file ($fasta_nr) with all non redundant sequences inclding originals and concatenated (summ of the first two)
my $orig_fas = $outfolder.'/orig.fas'; 
my $concat_fas = $outfolder.'/concat.fas'; 
# concat_boool = if conat seq exists, 0 otherwise
my $concat_bool = make_fasta_wo_100(\%seq, \%red, $fasta_nr, $orig_fas, $concat_fas);

#-------------------------------------------------------
#call blast concat_fas against _orig
   my $concat_orig_bl = $blast_out;
   $concat_orig_bl =~ s/m8/concat_orig_m8/;
$file_bool = check_if_first_line_empty($orig_fas);		
   call_blast_tbl_soft_filter_2files($concat_fas, $orig_fas, $concat_orig_bl, $e, $blast_path);
	if ($syst eq 'win' and $file_bool == 1)
	{
	    delete_blast_temp_files_win1($outfolder);
	}
	elsif ($syst eq 'linux'and $file_bool == 1)
	{
	    delete_blast_temp_files($outfolder);
	}

# autohits are not present car db and fasta file conain diff sequences IDEM pour reciproc hits seq1-seq2, seq2-seq1
# look at each line of the blast; put counter into a table if multiple hit.
# Then go through file again to eliminate autohits, multiple hits
# print file with only informatif lines (no rec, auto or multiple) and define @perc100 with lines that have 100 ident.

   my $blast_short_2 = $blast_short;
   $blast_short_2 =~ s/short/short_2/;	
   $multihit =~ s/\./_2\./; 		
   keep_info_lines_make_perc100($concat_orig_bl, $blast_short_2, $multihit, \@perc100);
#-------------------------------------------------------
#call blast concat_fas against concat_fas
   my $concat_concat_bl = $blast_out;
   $concat_concat_bl =~ s/m8/concat_concat_m8/;		
   call_blast_tbl_soft_filter_2files($concat_fas, $concat_fas, $concat_concat_bl, $e, $blast_path);
	if ($syst eq 'win' and $concat_bool == 1)
	{
	    delete_blast_temp_files_win1($outfolder);
	}
	elsif ($syst eq 'linux' and $concat_bool == 1)
	{
	    delete_blast_temp_files($outfolder);
	}

   eliminate_reciproc_hits($concat_concat_bl, $blast_non_rec);

# autohits are not present car db and fasta file conain diff sequences
# look at each line of the blast; put counter into a table if multiple hit.
# Then go through file agin to eliminate autohits, multiple hits
# print file with only informatif lines (no rec, auto or multiple) and define @perc100 with lines that have 100 ident.

   my $blast_short_3 = $blast_short;
   $blast_short_3 =~ s/short/short_3/;	
   $multihit =~ s/\./_3\./; 		
   keep_info_lines_make_perc100($blast_non_rec, $blast_short_3, $multihit, \@perc100);
#----------------------------------------------------------------------------

my $blast_short_final = $blast_short;
$blast_short_final =~ s/short/short_final/;

pool_blast_short_wo_red($blast_short, $blast_short_2, $blast_short_3, $blast_short_final, \%red);
print_hachage_into_file_1(\%red, $red_file);
print_hachage_into_file_1(\%concat_info, $concat_info_file);


if (0)
{
if ($syst eq 'win')
{
delete_files_motif_win($outfolder, 'non_rec*');
delete_files_motif_win($outfolder, '.tbl1*');
delete_files_motif_win($outfolder, 'overlap.fas');
delete_files_motif_win($outfolder, 'concat*');
delete_files_motif_win($outfolder, 'orig.fas');
delete_files_motif_win($outfolder, '_short_2*');
delete_files_motif_win($outfolder, '_short_3*');
delete_files_motif_win($outfolder, '_short.tbl');
delete_files_motif_win($outfolder, '_m8*');
}
elsif ($syst eq 'linux')
{
delete_files_motif($outfolder, 'non_rec*');
delete_files_motif($outfolder, '.tbl1*');
delete_files_motif($outfolder, 'overlap.fas');
delete_files_motif($outfolder, 'concat*');
delete_files_motif($outfolder, 'orig.fas');
delete_files_motif($outfolder, '_short_2*');
delete_files_motif($outfolder, '_short_3*');
delete_files_motif($outfolder, '_short.tbl');
delete_files_motif($outfolder, '_m8*');
}
}

# reads all files with multiple hits, if non of the seq is redundant put line into a new file, eliminate reciproc hits
make_multihit_pairs_non_red($outfolder, \%red, $multihit_final);

#deletes unneceray files
unless ( opendir(FOLDER, $outfolder) )
{
      print "Cannot access to folder $folder\n";
      exit;
}
my @filenames_del = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);
foreach my $file (@filenames_del)
{
    if ($file =~ /non_rec/ or $file eq 'concat.fas' or $file =~ /\.tbl1/ or $file =~ /overlap\.fas/ or $file=~ /orig\.fas/ or $file =~ /_short_2/ or $file =~ /_short_3/ or $file =~ /_short\.tbl/ or $file =~ /m8\.tbl/ or $file =~ /multihit\.tbl/ or $file =~ /multihit_2/)
    {
        delete_win_linux($file, $outfolder, $syst);
    }
}

#end if
}
# end foreach filename  
}

exit;

#####################################################################
sub select_perfect_cover_or_overlap
{

my ($line, $seq) =@_;
my $brin = '';
my $cover = '';


# $cover = 1 cover, query is shorter than sbj; 
#$cover = 2 cover, sbj is shorter than query 
#$cover = 3 overlap at the end of the query, beg of sbj
#$cover = 4 overlap at the end of the sbj, beg of query
#$cover = 5 overlap at the end of the query, end of sbj
#$cover = 6 overlap at the beg of the query, beg of sbj

#[0] query [1] sbj [2] %id;[3]HSP lenght;[4]mismath?;[5]gap?;[6]query_first;[7]query_last;[8]sbj first;[9]sbj_last;[10]e;[11]score;

# brin+
	    if ($$line[8] < $$line[9])
	    {
		$brin = 'p';
#query is shorter than $subj cover complete brin+
#print "$$line[6]\n";
#print "$$line[7]\n";
#print "$$seq{$$line[0]}\n";
#print "@$line\n";
		if (($$line[6] == 1) and ($$line[7] == $$seq{$$line[0]}))
		{
			$cover = 1;	
		}
#sbj is shorter than query cover complete brin+
		elsif ($$line[8] == 1 and $$line[9] == $$seq{$$line[1]})
		{
			$cover = 2;
		}
# overlap at the end of the query, beg of sbj
		elsif ($$line[7] == $$seq{$$line[0]} and $$line[8]==1)
		{
			$cover = 3;
		}
# overlap at the end of the sbj, beg of query
		elsif ($$line[6] == 1 and $$line[9]==$$seq{$$line[1]})
		{
			$cover = 4;
		}
		else
		{
			$cover = 7;
		}
	    }
#brin-
	    else
	    {
                $brin = 'm';
#query is shorter than $subj cover complete brin-
#print "@$line\n";
		if ($$line[6] == 1 and $$line[7] == $$seq{$$line[0]})
		{
			$cover = 1;
		}
#sbj is shorter than query cover complete brin-
		elsif ($$line[8] == $$seq{$$line[1]} and $$line[9] == 1)
		{
			$cover = 2;
		}
# overlap at the end of the query, end of sbj
                elsif ($$line[7] == $$seq{$$line[0]} and $$line[8]== $$seq{$$line[1]})
		{
			$cover = 5;
		}
# overlap at the beg of the query, beg of sbj
		elsif ($$line[6] == 1 and $$line[9]== 1)
		{
			$cover = 6;
		}
		else
		{
			$cover = 7;
		}

#end of else
	    }

my $x = $brin.$cover;
	    
return $x;

}
    
##############################################################

sub find_longest_red
{
my ($red) = @_;

my @short = keys %$red;
my %temp = ();


foreach my $short (@short)
{
    my $temp_short_seq = $short;
    while (exists $$red{$temp_short_seq})
    {
	$temp{$short} =  $$red{$temp_short_seq};
	$temp_short_seq = $$red{$temp_short_seq};
    }
}

%$red = ();
%$red = %temp;

#print_hachage(\%temp);
}

#####################################################

sub eliminate_reciproc_hits
{
my ($blast_out, $blast_non_rec) = @_;
	open_file_out($blast_non_rec, 'NREC');
	open_file_in($blast_out, 'BL');
	my %temp = ();
	while (my $line = <BL>)
	{
		my @line = split(' ', $line);
		unless (exists $temp{$line[1].'-'.$line[0]})
		{
			$temp{$line[0].'-'.$line[1]} = '';
			print NREC "$line";
		}
	}
	close (BL);
	close (NREC);
}
###########################################################

sub keep_info_lines_make_perc100
{
my ($blast_out, $blast_short, $multihit, $perc100) = @_;
my %multihit = ();
	
# look at each line of the blast; put counter into a table if multiple hit.
# Then go through file agin to eliminate autohits, multiple hits
# print file with only informatif lines (no rec, auto or multiple) and define @perc100 with lines that have 100 ident.

    open_file_in($blast_out, 'BL');
	my $prev_pair = '';
	my $count = 0;
	while (my $line = <BL>)
	{
		++$count;
		my @line = split(' ', $line);
# elimnate autohits
		unless ($line[0] eq $line[1])
		{
			my $pair = $line[0].'-'.$line[1];
			if ($pair eq $prev_pair)
			{
				unless (exists $multihit{$count-1})
				{
					$multihit{$count-1} = '';
				}
				$multihit{$count} = $line;
			}
			$prev_pair = $pair;
		}
	}
	close (BL);	
# go through blast lines again and copi only 'informativ lines' into new file (no autohit and multihit)
    open_file_in($blast_out, 'BL');
    open_file_out($blast_short, 'SHORT');	
    open_file_out($multihit, 'MULT');	
		
	$count = 0;
	while (my $line = <BL>)
	{
		++$count;
		my @line = split(' ', $line);
# elimnate autohits
		unless ($line[0] eq $line[1])
		{
			if (exists $multihit{$count})
			{
				print MULT "$line";
			}
			else
			{
# define table with onlt 100 identiy lines in it	
				if ($line[2] == 100)
				{
					push (@$perc100, $line);
				}
				print SHORT "$line";
			}
		}
	}
	%multihit = ();
	close (BL);	
	close (SHORT);
	close (MULT);
}
##################################################################

sub define_cover_red_overlap_table
{
my ($perc100, $seq_l, $red, $overlap) = @_;

	foreach my $line (@$perc100)
	{
		my @line = split(' ', $line);
# $cover = 1 cover, query is shorter than sbj; 
#$cover = 2 cover, sbj is shorter than query 
#$cover = 3 overlap at the end of the query, beg of sbj
#$cover = 4 overlap at the end of the sbj, beg of query
#$cover = 5 overlap at the end of the query, end of sbj
#$cover = 6 overlap at the beg of the query, beg of sbj

		my $cover = select_perfect_cover_or_overlap(\@line, \%$seq_l);
#           print "$cover ";
		$cover =~ /[pm]/;
		my $brin = $&;
		$cover =~ s/$brin//;
		if ($cover == 1) 
		{
# avoid defining red of both direction if it is the same length
			if (exists $$red{$line[1]})
			{	
				unless($$red{$line[1]} eq $line[0])
				{
                   		$$red{$line[0]} = $line[1]; 
				}
			}
			else
			{
				$$red{$line[0]} = $line[1]; 
			}
		}
		elsif ($cover == 2)
		{
                    $$red{$line[1]} = $line[0]; 
		}
		elsif ($cover < 7)
		{
			$line = $cover.';'.$line;
			push (@$overlap, $line);
		}
	}
}
###########################################################

sub complete_overlap_with_all_hits_for_all_code
{
my($over) = @_;

	foreach my $code (keys %$over)
	{
		my @hits = split(';', $$over{$code});
		foreach my $hit (@hits) 
		{
			foreach my $hit1 (@hits)
			{
				unless ($$over{$hit} =~ /$hit1;/)
				{
				  $$over{$hit} .= $hit1.';';
				}
			}
		}
	}
}
#################################################

sub concat_overlap_step_by_step
{
 my ($over, $seq, $outfolder, $blast_path, $e, $seq_l, $db_file) =@_;	
my @codes = sort keys %$over;
my $fas_file = $outfolder.'/fasta.fas';
my $blast_out = $outfolder.'/blast_m8.tbl';	
my %red = ();

# create intitial db file
	my $seq_code = shift (@codes);
	open_file_out($db_file, 'DB');

	print DB ">$seq_code\n$$seq{$seq_code}\n";
	close DB;

# blast each seq one by one againts the increasing database
       my $i = 0;
      foreach my $code (@codes)
	{
                ++$i;
#                print "$i ";
# if redundance is found booleen = 1, thus the program takes the next sequence
        my $booleen = 0;
# make a fasta file with the new sequence and blasts it againts the database
		open_file_out($fas_file, 'FAS');
		print FAS ">$code\n$$seq{$code}\n";
		close FAS;
		$blast_out = call_blast_m8($blast_path, $db_file, $fas_file, $e);

# sparse blast_out
# identify if 100 complete cover or overlap
		open_file_in($blast_out, 'BL');
		while (my $line = <BL>)
		{
                   	if ($booleen == 1)
                        {
                                next;
                        }
			my @line = split(' ', $line);	
			if ($line[2] == 100)
			{
                 #       print "$line\n";  
# $cover = 1 cover, query is shorter than sbj; 
#$cover = 2 cover, sbj is shorter than query 
#$cover = 3 overlap at the end of the query, beg of sbj
#$cover = 4 overlap at the end of the sbj, beg of query
#$cover = 5 overlap at the end of the query, end of sbj
#$cover = 6 overlap at the beg of the query, beg of sbj

				my $cover = select_perfect_cover_or_overlap(\@line, \%$seq_l);
				$cover =~ /[pm]/;
				my $brin = $&;
				$cover =~ s/$brin//;
				if ($cover == 1)
				{
					$red{$line[0]} = $line[1];
                              $booleen = 1;
				}
				elsif($cover == 2)
				{
					$red{$line[1]} = $line[0];
# rewrites db file : adds query, deletes sbj
                              add_1seq_delete_1seq_file($db_file, \%$seq, $line[0], $line[1]);                          
                              $booleen = 1;
                         }
#overlap 100 id
                        elsif ($cover>2 and $cover <7)
                         {
                              concat_overlap_from_blast_line(\@line, $cover, \%$seq, \%$seq_l); 
                     #        print "$cover\n$line\n$seq{$line[0]}\n$seq{$line[1]}\n$seq{$line[0].'-'.$line[1]}\n";         
                              $red{$line[1]} = $line[0].'-'.$line[1];
                              $red{$line[0]} = $line[0].'-'.$line[1];
                              add_1seq_delete_1seq_file($db_file, \%$seq, $line[0].'-'.$line[1], $line[1]);
                              $booleen = 1;
                         }
# end if 100%
			}
	# end while
		}
		close (BL);	
#adds sequence to database if no redundancy was observed
                if ($booleen == 0)
                {
                     add_query_to_db($db_file, $fas_file, $db_file);
                }

# end foreach my $code
	}

return %red;

}

#######################################################
sub add_1seq_delete_1seq_file
{
my ($db_file, $seq, $new_seq, $old_seq) = @_;
					
my %seq_temp = read_hach_from_fasta_code_wo_space($db_file);
my @codes = sort keys %seq_temp;
			
open_file_out($db_file, 'DB');

foreach my $code (@codes)
{
	unless ($code eq $old_seq)
	{
                print DB ">$code\n$seq_temp{$code}\n";
	}
}
print DB ">$new_seq\n$$seq{$new_seq}\n";

close (DB);
}
##########################################################

sub concat_overlap_from_blast_line
{
my ($line, $cover, $seq, $seq_l) = @_;

# $cover = 1 cover, query is shorter than sbj; 
#$cover = 2 cover, sbj is shorter than query 
#$cover = 3 overlap at the end of the query, beg of sbj
#$cover = 4 overlap at the end of the sbj, beg of query
#$cover = 5 overlap at the end of the query, end of sbj
#$cover = 6 overlap at the beg of the query, beg of sbj


# query end subj beg
if ( $cover == 3)
{
#    print "@$line\n";
    my $fragment = substr($$seq{$$line[0]}, 0, $$line[6]-1);
    $$seq{$$line[0].'-'.$$line[1]} = $fragment.$$seq{$$line[1]};
    $$seq_l{$$line[0].'-'.$$line[1]} = length $$seq{$$line[0].'-'.$$line[1]};
}  
# query beg subj end
elsif ($cover == 4)
{
#    print "@$line\n";
    my $fragment = substr($$seq{$$line[1]}, 0, $$line[8]-1);
    $$seq{$$line[0].'-'.$$line[1]} = $fragment.$$seq{$$line[0]};
    $$seq_l{$$line[0].'-'.$$line[1]} = length $$seq{$$line[0].'-'.$$line[1]};
}  
# query end subj end brin-
elsif ($cover == 5)
{
#    print "@$line\n";
    my $fragment = substr($$seq{$$line[0]}, 0, $$line[6]-1);
    my $rev_comp_subj = reverse_complement($$seq{$$line[1]});
    $$seq{$$line[0].'-'.$$line[1]} = $fragment.$rev_comp_subj;
    $$seq_l{$$line[0].'-'.$$line[1]} = length $$seq{$$line[0].'-'.$$line[1]};
} 
# query beg subj beg brin- 
elsif ($cover == 6)
{
#    print "@$line\n";
    my $fr_l = (length$$seq{$$line[1]}) - $$line[8] +1;
    my $fragment = substr($$seq{$$line[1]}, $$line[8], $fr_l);
    my $rev_comp_subj = reverse_complement($fragment);
    $$seq{$$line[0].'-'.$$line[1]} = $rev_comp_subj.$$seq{$$line[0]};
    $$seq_l{$$line[0].'-'.$$line[1]} = length $$seq{$$line[0].'-'.$$line[1]};
}  
  
}

####################################################################

sub add_query_to_db
{
my ($db_file_in, $fas_file, $db_file) = @_;

open_file_in($db_file_in, 'IN');
my @db = <IN>;
close (IN);
open_file_out($db_file, 'OUT');
foreach my $db (@db)
{
print OUT "$db";
}


open_file_in($fas_file, 'FAS');
my @fas = <FAS>;
close (FAS);
foreach my $fas (@fas)
{
print OUT "$fas";
}

close (OUT);
}

################################################

sub pool_hach
{
	my ($red, $red_overlap) = @_;	
	
	foreach my $code (keys %$red_overlap)
	{
		$$red{$code} = $$red_overlap{$code};
	}	
}
###############################################

sub make_fasta_wo_red_with_concat
{
my ($seq, $red, $db_file, $nonred_fasta) = @_;

my %db = read_hach_from_fasta_code_wo_space($db_file);

#writes a file with all nonred seqs plus the resuls of concat.
#%seq all original inretrmedier and finle concat seq

open_file_out($nonred_fasta, 'FAS');

foreach my $code (sort keys %$seq)
{
	unless (exists $$red{$code})
	{
		unless($code=~ /-/)
		{
		print FAS ">$code\n$$seq{$code}\n";
		}
	}
}

foreach my $code (keys %db)
{
	if ($code =~ /-/)
	{
		print FAS ">$code\n$db{$code}\n";
	}
}

close (FAS);


}

###################################################

sub concat_2seq
{
my ($line, $seq, $seq_l, $concat_count, $concat_info) = @_;

my $cover = $$line[12];

# $cover = 1 cover, query is shorter than sbj; 
#$cover = 2 cover, sbj is shorter than query 
#$cover = 3 overlap at the end of the query, beg of sbj
#$cover = 4 overlap at the end of the sbj, beg of query
#$cover = 5 overlap at the end of the query, end of sbj
#$cover = 6 overlap at the beg of the query, beg of sbj


# query end subj beg
if ( $cover == 3)
{
#    print "@$line\n";
    my $fragment = substr($$seq{$$line[0]}, 0, $$line[6]-1);
    ++$$concat_count;
#	print "$$concat_count ";
    $$seq{'concat'.$$concat_count} = $fragment.$$seq{$$line[1]};
    $$seq_l{'concat'.$$concat_count} = length $$seq{'concat'.$$concat_count};
	$$concat_info{'concat'.$$concat_count} = $$line[0].';'.$$line[1];
}  
# query beg subj end
elsif ($cover == 4)
{
#    print "@$line\n";
    my $fragment = substr($$seq{$$line[1]}, 0, $$line[8]-1);
    ++$$concat_count;
#	print "$$concat_count ";
    $$seq{'concat'.$$concat_count} = $fragment.$$seq{$$line[0]};
    $$seq_l{'concat'.$$concat_count} = length $$seq{'concat'.$$concat_count};
	$$concat_info{'concat'.$$concat_count} = $$line[0].';'.$$line[1];

}  
# query end subj end brin-
elsif ($cover == 5)
{
#    print "@$line\n";
    my $fragment = substr($$seq{$$line[0]}, 0, $$line[6]-1);
    my $rev_comp_subj = reverse_complement($$seq{$$line[1]});
    ++$$concat_count;
#	print "$$concat_count ";
    $$seq{'concat'.$$concat_count} = $fragment.$rev_comp_subj;
    $$seq_l{'concat'.$$concat_count} = length $$seq{'concat'.$$concat_count};
	$$concat_info{'concat'.$$concat_count} = $$line[0].';'.$$line[1];

} 
# query beg subj beg brin- 
elsif ($cover == 6)
{
#    print "@$line\n";
    my $fr_l = (length$$seq{$$line[1]}) - $$line[8] +1;
    my $fragment = substr($$seq{$$line[1]}, $$line[8], $fr_l);
    my $rev_comp_subj = reverse_complement($fragment);
    ++$$concat_count;
#	print "$$concat_count ";
    $$seq{'concat'.$$concat_count} = $rev_comp_subj.$$seq{$$line[0]};
    $$seq_l{'concat'.$$concat_count} = length $$seq{'concat'.$$concat_count};
	$$concat_info{'concat'.$$concat_count} = $$line[0].';'.$$line[1];
}  

return $$seq{'concat'.$$concat_count};

}
###################################################

sub make_file_overlap_remain
{
my ($overlap, $over_fasta, $seq, $concat, $red) = @_;

foreach my $line (@$overlap)
{
	my @line = split(' ', $line);
#	print "$line[0] $line[1]\n";
	unless(exists $$red{$line[0]})
	{
		unless(exists $$concat{$line[0]})
		{
			$$concat{$line[0]} = $$seq{$line[0]};
		}
	}
	unless(exists $$red{$line[1]})
	{
		unless(exists $$concat{$line[1]})
		{
			$$concat{$line[1]} = $$seq{$line[1]};
		}
	}


}

print_hachage_into_fasta(\%$concat, $over_fasta);

}

########################################################

sub eliminate_red_from_overlap
{
my ($overlap, $red) = @_;
my @overlap_wored = ();
       for( my $i= 0; $i < scalar@$overlap; ++$i)
       {
		my $line = $$overlap[$i];
#	print "$line";
		$line =~ /.+;/;
		my $cover = $&;
                $line =~ s/$cover//;
		$cover =~ s/;//;
                chomp $line;
                $line .= ' '.$cover."\n";
 #               print "$line\n";
		$$overlap[$i] = $line;
 		my @line = split(' ', $line);
		if (exists $$red{$line[0]} or exists $$red{$line[1]})
		{
			next;
		}
		else
		{	
			push (@overlap_wored, $line);		
#			$over{$line[0]} .= $line[1].';';
#			$over{$line[1]} .= $line[0].';';
		}				
       }

	@$overlap = ();
	@$overlap = @overlap_wored;
	@overlap_wored = ();
#       print "*****************\n@$overlap\n***************\n";


#	print_hachage(\%over);
#	print_hachage_into_file_1(\%over, 'temp1/over.tbl');
}

#############################################################

sub make_fasta_wo_100
{
my ($seq, $red, $fasta_nr, $orig_fas, $concat_fas) = @_;
open_file_out($fasta_nr, 'FAS');
open_file_out($orig_fas, 'ORI');
open_file_out($concat_fas, 'CON');

my @codes = sort keys %$seq;
my $bool = 0;
foreach my $code (@codes)
{
	unless(exists $$red{$code})
	{
		print FAS ">$code\n$$seq{$code}\n";
		if ($code =~ /concat/)
		{
			print CON ">$code\n$$seq{$code}\n";	
			$bool = 1;
		}
		else
		{
			print ORI ">$code\n$$seq{$code}\n";	
		}
	}
}

close (FAS);
close (ORI);
close (CON);
return $bool;
}

#######################################################

sub concat_overlap_in_cycle
{
my ($concat_info, $concat_count, $overlap, $red, $seq, $seq_l, $outfolder, $blast_out, $multihit, $blast_short, $blast_non_rec, $e, $blast_path) = @_;

# while there are lines in overlap, concat sequences if non of the pair is red, 
# makes new file with concat seq, and the noncatanated, but overlaping sequences, and blast, sort redundant, make new @overlap

my $over_round = 0;
my @overlap_wored;
while(@$overlap)
{
	++$over_round;
        print "over_round: $over_round\n";
	my %concat = ();

	foreach my $line (@$overlap)
	{
		my @line = split(' ', $line);
		if ((exists $$red{$line[0]}) or (exists $$red{$line[1]}))
		{
			push(@overlap_wored, $line);
		}
		else
		{
			my $new_seq = concat_2seq(\@line, \%$seq, \%$seq_l, \$$concat_count, \%$concat_info);
                  $concat{'concat'.$$concat_count} = $new_seq;
			$$red{$line[0]} = 'concat'.$$concat_count;
			$$red{$line[1]} = 'concat'.$$concat_count;
		}
	}
	my $over_fasta = $outfolder.'/overlap.fas';
	make_file_overlap_remain(\@overlap_wored, $over_fasta, \%$seq, \%concat, \%$red);
#	print "**************\n@overlap_wored\n********************\n";
	
	$blast_out .= $over_round; 
   call_blast_tbl_soft_filter($over_fasta, $blast_out, $e, $blast_path);
	if ($syst eq 'win')
	{
	    delete_blast_temp_files_win1($outfolder);
	}
	elsif ($syst eq 'linux')
	{
	    delete_blast_temp_files($outfolder);
	}

	$blast_non_rec .= $over_round; 
	eliminate_reciproc_hits($blast_out, $blast_non_rec);
	$multihit .= $over_round;
	$blast_short .= $over_round;
	my @perc100 = ();
	keep_info_lines_make_perc100($blast_non_rec, $blast_short, $multihit, \@perc100);
#	print "**************\n@perc100\n********************\n";
# make hach with short-long complete cover %red
# put perfect overlap into new table @overlap
        @$overlap = ();
	define_cover_red_overlap_table(\@perc100, \%$seq_l, \%$red, \@$overlap);
#	print "**************\n@overlap\n********************\n";
# for each redundant find the longest idem
        find_longest_red(\%$red);
#	print_hachage(\%$red);
eliminate_red_from_overlap(\@$overlap, \%$red);
#	print "**************\n@overlap\n********************\n";

# print "@$overlap\n";

# end while @overlap
}

}

############################################################################

sub pool_blast_short_wo_red
{
my ($blast_short, $blast_short_2, $blast_short_3, $blast_short_final, $red) = @_;
open_file_out($blast_short_final, 'OUT');
open_file_in($blast_short, 'IN');

while(my $line = <IN>)
{
	my @line =split(' ', $line);
	unless (exists $$red{$line[0]})
	{
		unless (exists $$red{$line[1]})
		{
			print OUT "$line";
		}
	}
}
close (IN);

open_file_in($blast_short_2, 'IN');
while(my $line = <IN>)
{
	print OUT "$line";
}
close (IN);
open_file_in($blast_short_3, 'IN');
while(my $line = <IN>)
{
	print OUT "$line";
}
close (IN);

close (OUT);

}

#########################################################################

sub make_multihit_pairs_non_red
{
my ($outfolder, $red, $hit) = @_;
my %temp = ();

  unless ( opendir(FOLDER, $outfolder) )
  {
      print "Cannot access to folder $outfolder\n";
      exit;
  }
  my @filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
 closedir(FOLDER);

open_file_out($hit, 'HIT');
	foreach my $file (@filenames)
	{
		if ($file =~ /multihit/)
		{
			if ($file =~ /final/)
			{
				next;
			}
			$file = $outfolder.'/'.$file;
			open_file_in($file, 'IN');
			while(my $line = <IN>)
			{
				my @line = split(' ', $line);
				unless(exists $$red{$line[0]})
				{
					unless (exists $$red{$line[1]})
					{
						unless (exists $temp{$line[1].'-'.$line[0]})
						{
							print HIT "$line";
							$temp{$line[0].'-'.$line[1]} = 1;
						}
						
					}
				}
			}
			close (IN);

		# end if 
		}
	}
close (HIT);
%temp = ();

	
}
