#!/usr/bin/perl -w

# Prg: select_unique_make_cons.pl

# INPUT: 	xxx_nr.fas fasta file with only non_redundant sequences in it
#		xxx.fas (original fasta file)
#		xxx_all_info.txt (output of all_seq_gr_info.pl)

#only 1 series of files in the input folder at a time 
# make a fasta file with unique sequeces (cat1 nohit, Cat2 redundant)
# takes all sequences cat 4 at most, makes groups; makes 2/3 consensus sequences
# males a file with consensus sequences + all seq. that are cat. 5 (not included in cons)

# OUTPUT: 	xxx_unique.fas  unique sequeces (cat1 nohit + Cat2 redundant)
#		xxx_cons_plus_gr.fas (all consensus seq + all Cat5)

  use warnings;
  use strict;
  use subprogramEM;

my $folder = '';
my $outfolder = '';
my $syst = '';
my $Eblast = 0;
my $blast_path = '';
# number of indiv. used for the sequenceing; if unknown, put 9999;
my $numb_ind = 0;
my $clustal_path = '';
my $clust_ver = 0;
my $prop_maj = 0;


if (scalar@ARGV == 8)
{
	$folder = $ARGV[0];
	$outfolder = $ARGV[1];
	$syst = $ARGV[2];
	$Eblast = $ARGV[3];
	$blast_path = $ARGV[4];
	$clustal_path = $ARGV[5];
	$clust_ver = $ARGV[6];
	$prop_maj = $ARGV[7];
}
else
{
	$folder = 'datatemp';
	$outfolder = 'temp';
#	$syst = 'linux';
	$syst = 'win';
	$Eblast = '1E-40';
	$blast_path = 'c:/BLAST2_2_18/bin/';
#	$blast_path = '/home/user0/bin/blast-2.2.19/bin/';
	$clustal_path = 'c:/CLUSTALW/clustalw2/';
#	$clustal_path = '/home/user0/bin/clustalw-2.0.10/';
	$clust_ver = 2;
	$prop_maj = 0.66;

}

my $clust_inp_file = $outfolder.'/clust';
my $script_file = $outfolder.'/script_file';
my $clust_out = $clust_inp_file.'.aln';
make_script_file($script_file, $clust_inp_file);

 unless ( opendir(FOLDER, $folder) )
  {
      print "Cannot access to folder $folder\n";
      exit;
  }
my @filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);
my $fas_file = '';
my $fas_orig = '';
my $info_file = '';
my $fas_unique = ''; 
my $fas_cons = ''; 
my $alignfas = ''; 
#my $concat_file = '';
my $gr_file = '';

# define filenames
foreach my $filename (@filenames)
{

	if ($filename =~ /_nr\.fas$/)
	{
		$fas_unique = $filename;
		$fas_unique =~ s/\.fas$/_unique\.fas/;
		$fas_unique = $outfolder.'/'.$fas_unique;

		$fas_cons = $filename;
		$fas_cons =~ s/\.fas$/_cons_plus_gr\.fas/;
		$fas_cons = $outfolder.'/'.$fas_cons;

		$alignfas = $filename;
		$alignfas =~ s/\.fas$/_align\.fas/;
		$alignfas = $outfolder.'/'.$alignfas;

	$filename = $folder.'/'.$filename;

		$gr_file = $fas_cons;
		$gr_file =~ s/_cons_plus_gr\.fas$/_groups\.txt/;

		$fas_file = $filename;
		$fas_orig = $filename;
		$fas_orig =~ s/_mask_nr//;
	}
	elsif ($filename =~ /_all_info\.txt$/)
	{
	$filename = $folder.'/'.$filename;
		$info_file = $filename;
#		print "$info_file\n";
	}
}
#print "$info_file\n$fas_orig\n$fas_file\n";

my %nohit = ();


my %seq = read_hach_from_fasta_code_wo_space($fas_file);
my %seq_orig = read_hach_from_fasta_code_wo_space($fas_orig);

open_file_in($info_file, 'IN');
my $title = <IN>;
open_file_out($fas_unique, 'UNI');

#############
my $t = 0;
my %cat = ();
# make a hash with all cat 3 and 4 sequences as a code and all hits to these sequences as a value
my %sel = ();

while (my $line = <IN>)
{
	if ($line eq "\n")
	{
		next;
	}
	my @line = split(';', $line);
# make a fasta file with nohit and redundant only sequences (no multihit)
	if ($line[6] < 3 and $line[5] eq '')
	{
		print UNI ">$line[0]\n$seq{$line[0]}\n";
	}

# select sequences cat 3 and 4 (ms allele or flankig region similarity is above set level) 
# make a category hach; if multihit, category= 6
	if ($line[5] eq '')
	{
		$cat{$line[0]} = $line[6];
	}
	else
	{
		$cat{$line[0]} = 6;
	}

	if ($cat{$line[0]} == 4 or $cat{$line[0]} == 3)
	{
		++$t;
		$sel{$line[0]} = $line[0].','.$line[1].','.$line[2].','.$line[3].',';
		$sel{$line[0]} =~ s/,,+/,/g;
	}
# end while
}
close IN;
close UNI;
print "$t\n";
#print_hachage(\%sel);

#############
# eliminate sequene codes that have category higer than 4
if (1)
{
my %sel_short = ();
foreach my $sel (keys %sel)
{
	my @sel_temp = split(',', $sel{$sel});
	foreach my $sel_temp (@sel_temp)
	{
		if (exists $cat{$sel_temp})
		{
			if ($cat{$sel_temp} > 4 )
			{
				$sel{$sel} =~ s/$sel_temp,//;
			} 
		}
	}
	@sel_temp = split(',', $sel{$sel});
	unless (scalar @sel_temp == 1)
	{
		$sel_short{$sel} = $sel{$sel};
	}	
}
%sel = %sel_short;
%sel_short = ();
$t = scalar(keys %sel);
print "$t\n";

}

#print_hachage(\%sel);
#############
# make groups in the largest sense with sequences (if A is sim to B and B is sim to C than A is sim to C)
my %gr = ();
my $i = 1;
my @codes = sort keys %sel;
if (scalar@codes > 0)
{
$gr{1} = $sel{shift@codes};
#print "$gr{1}\n";
foreach my $code (keys %sel)
{
	unless ($sel{$code} eq '')
	{
		my @sel = split(',', $sel{$code});
		my $bool_gl = 0;
		foreach my $gr (keys %gr)
		{
		my $booleen = 0;
			foreach my $sel(@sel)
			{
				if ($gr{$gr} =~ /$sel,/)
				{
					$booleen = 1;
				}
			}
			if ($booleen == 1)
			{
				foreach my $sel(@sel)
				{
					unless ($gr{$gr} =~ /$sel,/)
					{
						$gr{$gr} .= $sel.',';
					}
				}
				$bool_gl = 1;
			}
		}
		if ($bool_gl ==0)
		{
			++$i;
			$gr{$i} = $sel{$code}
		}

	}
}
#print_hachage(\%gr);
}

#############
# makes a consesnsus sequence for each group
open_file_out($gr_file, 'GR');
open_file_out($alignfas, 'FAS');
# sequences that are included in a consensus sequence
my %aligned_seq = ();
# consesnsus sequences
my %cons = ();
my $no_orient = ''; 
foreach my $gr (sort {$a<=>$b}keys %gr)
{
# makes a fasta file with all the sequences in the group
	my @temp = split(',', $gr{$gr});
	open_file_out($clust_inp_file, 'CL');
	my $align_count = 0;
	foreach my $temp (@temp)
	{
		if ($temp =~ /concat/)
		{
			if (exists $seq{$temp})
			{
				print CL ">$temp\n$seq{$temp}\n";
				++$align_count;
			}
			else
			{
				$gr{$gr} =~ s/$temp,//;
			}
		}
		else 
		{
			print CL ">$temp\n$seq_orig{$temp}\n";
			++$align_count;
		}
	}
	close CL;
	if ($align_count > 1)
	{
	my $blast_out = $outfolder.'/blast_out';
# make a blast to determine relative orientation of the sequences
	call_blast_tbl_soft_filter($clust_inp_file, $blast_out, $Eblast, $blast_path);
# takes the reverse complement of the sequence if necessary (first seq in the list is the basis)
	my $no_orient_temp = rev_comp_blast($clust_inp_file, $blast_out);
	$no_orient .= $no_orient_temp;
	my @no_orient_temp = split(';', $no_orient_temp);
	if (($align_count - scalar@no_orient_temp) > 1)
	{
# run clustalW
	    if ($clust_ver == 2)
	    {
		call_clustalw2_path($script_file, $clustal_path, $outfolder);
	    }
	    else
	    {
		call_clustalw_path($script_file, $clustal_path, $outfolder);		
	    }
#extract aligned sequences
		my %seq_align = ();
	    extract_aligned_seq_mult($clust_out, \%seq_align);
# make a consensus sequences $cons_N => if difference on site, put N
		my $cons = make_cons_maj(\%seq_align, $prop_maj);

		my $bool_vec = 0;
		foreach my $seq_al (sort keys %seq_align)
		{
			print FAS ">$seq_al\n$seq_align{$seq_al}\n";
	#		$aligned_seq{$seq_al} = $seq_align{$seq_al};
			$aligned_seq{$seq_al} = '';
			if ($seq_al =~ /_N$/)
			{
				$bool_vec = 1;
			}
		}
		my $n = scalar @temp;
		my $code_temp = 'cons'.$gr.'_'.$n;
		if ($bool_vec ==1)
		{
			$code_temp .= '_N';
		}
		else
		{
			$code_temp .= '_A';
		}
		$cons{$code_temp} = $cons;
		print FAS ">$code_temp\n$cons\n";
# end if $align_count - scalar @no_orient_temp
	}
	else
	{
		$gr{gr} = '';
	}
# end if $align_count
	}
	print GR "$gr;$gr{$gr}\n";
# end foreach my $gr
}
close GR;
close FAS;

###########
# make a fasta file with all nr sequences that are ((i) cat 5 (ii) not multihit 
# (iii) not included in the conc$sensus sequences )+ consensus sequences + if non_oriented sequenes

open_file_out($fas_cons, 'FASO');
foreach my $seq (sort keys %cat)
{
	if ($cat{$seq} == 5)
	{
		unless (exists $aligned_seq{$seq})
		{
			print FASO ">$seq\n$seq{$seq}\n";
		}
	}
}
foreach my $seq (sort keys %cons)
{
	print FASO ">$seq\n$cons{$seq}\n";
}

my @nonor = split(';', $no_orient);
foreach my $seq (sort @nonor)
{
	if (exists $seq{$seq})
	{ 
		print FASO ">$seq\n$seq{$seq}\n";
	}
	else
	{ 
		print FASO ">$seq\n$seq_orig{$seq}\n";
	}

}

close FASO;

if (0)
{
# delete non_essential outfiles
my $motdel = '';
if ($syst eq 'win')
{
	delete_blast_temp_files_win1($outfolder);
	$motdel = 'script_file';
	delete_files_motif_win($outfolder, $motdel);
	$motdel = 'clust';
#	delete_files_motif_win($outfolder, $motdel);
	$motdel = 'clust.aln';
	delete_files_motif_win($outfolder, $motdel);
	$motdel = 'clust.dnd';
	delete_files_motif_win($outfolder, $motdel);
	$motdel = 'clustout';
	delete_files_motif_win($outfolder, $motdel);
	$motdel = 'blast_out';
	delete_files_motif_win($outfolder, $motdel);
}
elsif ($syst eq 'linux')
{
	delete_blast_temp_files1($outfolder);
	$motdel = 'script_file';
	delete_files_motif($outfolder, $motdel);
	$motdel = 'clust';
#	delete_files_motif($outfolder, $motdel);
	$motdel = 'clust.aln';
	delete_files_motif($outfolder, $motdel);
	$motdel = 'clust.dnd';
	delete_files_motif($outfolder, $motdel);
	$motdel = 'clustout';
	delete_files_motif($outfolder, $motdel);
	$motdel = 'blast_out';
	delete_files_motif($outfolder, $motdel);
}
}
#read all file in the outfolder and deletes then is unnecessary
unless ( opendir(FOLDER, $outfolder) )
{
      print "Cannot access to folder $folder\n";
      exit;
}
my @filenames_del = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);
foreach my $file (@filenames_del)
{
    if ($file eq 'script_file' or $file eq 'clustout' or $file =~ /clust\./ or $file =~ /blast_out/)
    {
	delete_win_linux($file, $outfolder, $syst);
    }
}


exit;

##############################################################

sub make_script_file
{
my ($sript_file, $clust_inp_file) = @_;

open_file_out($sript_file, 'SC');

print SC "1\n$clust_inp_file\n2\n1\n\n\nX\nX\nX\nX\n";

close (SC);
}

##############################################################

sub extract_aligned_seq_mult
{
my ($clust_out, $seq_align) = @_;
open_file_in($clust_out, 'CLI');
 my @data = <CLI>;
close (CLI);

shift@data;
foreach my $line (@data)
{
	if ($line =~ /^[^\s]+/ )
	{
		my $code = $&;
		$line =~ s/$code//;
		chomp $line;
		$line =~ s/\s//g;
		$$seq_align{$code} .= $line;
	}
}
}


####################################################

sub make_cons_N
{
my ($seq_align) = @_;
my $cons = '';

my @codes = sort keys %$seq_align;
my $align_l = length $$seq_align{$codes[0]};

foreach my $code (@codes)
{
	if ($$seq_align{$code} =~ /^-+/)
	{
		my $beg = $&;
		$beg =~ s/-/:/g;
		$$seq_align{$code} =~ s/^-+//;
		$$seq_align{$code} = $beg.$$seq_align{$code};
	}
	if ($$seq_align{$code} =~ /-+$/)
	{
		my $end = $&;
		$end =~ s/-/:/g;
		$$seq_align{$code} =~ s/-+$//;
		$$seq_align{$code} = $$seq_align{$code}.$end;
	}

}

for (my $i = 0; $i < $align_l; ++$i)
{
	my %site = ();
	foreach my $code (@codes)
	{
		my $base = substr($$seq_align{$code},$i,1);
		++$site{$base};
	}
	my @bases = sort keys %site;
	if (scalar @bases == 1 )
	{
		$cons .= $bases[0];
	}
	elsif (exists $site{':'} and scalar @bases == 2 )
	{
		$cons .= $bases[1];
	}
	else
	{
		$cons .= 'N';
	}
	
}

return $cons;
}

####################################################
sub make_cons_maj
{
# if the number of bases in a given site is greater than the 2/3 of the number of informative site, 
#takes the most frequent base else, writes N
my ($seq_align, $prop_maj) = @_;
my $cons = '';

my @codes = sort keys %$seq_align;
my $align_l = length $$seq_align{$codes[0]};

foreach my $code (@codes)
{
	if ($$seq_align{$code} =~ /^-+/)
	{
		my $beg = $&;
		$beg =~ s/-/:/g;
		$$seq_align{$code} =~ s/^-+//;
		$$seq_align{$code} = $beg.$$seq_align{$code};
	}
	if ($$seq_align{$code} =~ /-+$/)
	{
		my $end = $&;
		$end =~ s/-/:/g;
		$$seq_align{$code} =~ s/-+$//;
		$$seq_align{$code} = $$seq_align{$code}.$end;
	}

}

for (my $i = 0; $i < $align_l; ++$i)
{
	my %site = ();
	my $inf_site = 0;
	foreach my $code (@codes)
	{
		my $base = substr($$seq_align{$code},$i,1);
		unless ($base eq ':')
		{
			++$site{$base};
			++$inf_site;
		}
	}
	my @bases = sort keys %site;
	my $bool = 0;
	foreach my $base (@bases)
	{
		if ($site{$base} > ($inf_site * $prop_maj))
		{
			unless($base eq '-')
			{
				$cons .= $base;
			}
			$bool = 1;
		}
	}
	if ($bool ==0)
	{
		$cons .= 'N';
	}	
}

return $cons;
}

####################################################

sub rev_comp_blast
{

# determines orintaion of the sequences based on the blast_out file. 
#Occasionally some of the sequences dos not have hits to others in the group, 
#and thus their orientation is undefined. In this case the non-oriented sequences are not printed into the 
#clust_inp_file, their codes are returned, and they will be included in the cons_plus_group file separatelly

my ($clust_inp_file, $blast_out) = @_;
open_file_in($blast_out, 'BL');
my %seq = read_hach_from_fasta_code_wo_space($clust_inp_file);
my @codes = sort keys %seq;
my $non_orient = '';
	
my %orient = ();
while (my $line = <BL>)
{
	my $mot = '';
	my @line = split(' ', $line);
	if ($line[9] - $line[8] > 0)
	{
		$mot = $line[0].'-'.$line[1];
		$orient{$mot} = 1;
 	}
	else
	{
		$mot = $line[0].'-'.$line[1];
		$orient{$mot} = -1;
	}
}
close BL;

#print_hachage(\%orient);

my @pairs = sort keys %orient;
my %orient_seq = ();
$orient_seq{$codes[0]} = 1;

my $count = 0;
while (scalar keys %orient_seq < scalar @codes and $count < 11)
{
	++$count;
	print "$count ";
	define_orient(\@pairs, \%orient, \%orient_seq);
}

if ($count == 10)
{
	my $diff = scalar @codes - scalar keys %orient_seq;
#	print "$diff seq is not oriented!!!";
}

open_file_out($clust_inp_file, 'CLUS');
foreach my $code (@codes)
{
	if (exists $orient_seq{$code})
	{
		if ($orient_seq{$code} == -1)
		{
			$seq{$code} = reverse_complement($seq{$code});
		}
		print CLUS ">$code\n$seq{$code}\n";
	}
	else
	{
#		print "$code orientation is not defined\n";
		$non_orient .= $code.';';
	}
}	 
close CLUS;
return $non_orient;
}



#########################################################################

sub define_orient
{
my ($pairs, $orient, $orient_seq) = @_;
foreach my $or (@$pairs)
{
	my @ori = split('-', $or);
	if(exists $$orient_seq{$ori[0]})
	{
		if (exists $$orient_seq{$ori[1]})
		{
			next;
		}
	}
	else
	{
		if (exists $$orient_seq{$ori[0]})
		{
			$$orient_seq{$ori[1]} = $$orient{$or} * $$orient_seq{$ori[0]};
		}
		elsif (exists $$orient_seq{$ori[1]} )
		{
			$$orient_seq{$ori[0]} = $$orient{$or}*$$orient_seq{$ori[1]};
		}
	}
}
}
